#!/usr/bin/python -E
#
# build-policy - Build a basic policy using the Reference Policy 
#                security_classes, access_vector and initial_sids files.
#
# When pointed at the Reference Policy tree, builds a basic policy containing 
# the kernel and userspace object manager classes, permissions and initial 
# SIDs. It has two users defined (unconfined_u and system_u) with one type
# (unconfined_t). There are two roles defined (unconfined_r and object_r).
#
# The program will output either CIL or policy language formats suitable for
# building basic non-MLS and MLS binary policies.
#
# This will also build a header file containing the class/permission mapping
# structure for selinux_set_mapping(3) for the:
# 	security_class_to_string_with_class_mapping_example
# shown in the libselinux/example-code section.
#
#
# Note that the code has been adapted from the Python script "flask.py" written
# by Caleb Case <ccase@tresys.com> that is supplied with the Reference Policy
# to build the libselinux header files.
#

import getopt
import os
import sys
import re

# These are the policy user:role:type context entries:
se_user = "unconfined_u"
se_role = "unconfined_r"
se_type = "unconfined_t"
sys_user = "system_u"
obj_role = "object_r"

class ParseError(Exception):
	def __init__(self, type, file, line):
		self.type = type
		self.file = file
		self.line = line
	def __str__(self):
		typeS = self.type
		if type(self.type) is not str: typeS = Flask.CONSTANT_S[self.type]
		return "Parse Error: Unexpected %s on line %d of %s." % (typeS, self.line, self.file)

class DuplicateError(Exception):
	def __init__(self, type, file, line, symbol):
		self.type = type
		self.file = file
		self.line = line
		self.symbol = symbol
	def __str__(self):
		typeS = self.type
		if type(self.type) is not str: typeS = Flask.CONSTANT_S[self.type]
		return "Duplicate Error: Duplicate %s '%s' on line %d of %s." % (typeS, self.symbol, self.line, self.file)

class UndefinedError(Exception):
	def __init__(self, type, file, line, symbol):
		self.type = type
		self.file = file
		self.line = line
		self.symbol = symbol
	def __str__(self):
		typeS = self.type
		if type(self.type) is not str: typeS = Flask.CONSTANT_S[self.type]
		return "Undefined Error: %s '%s' is not defined but used on line %d of %s." % (typeS, self.symbol, self.line, self.file)

class UnusedError(Exception):
	def __init__(self, info):
		self.info = info
	def __str__(self):
		return "Unused Error: %s" % self.info

class Flask:
	'''
	FLASK container class with utilities for parsing definition
	files and creating source policy files (CIL and policy language versions).
	'''

	#Constants used in definitions parsing.
	WHITE    = re.compile(r'^\s*$')
	COMMENT  = re.compile(r'^\s*#')
	USERFLAG = re.compile(r'# userspace')
	CLASS    = re.compile(r'^class (?P<name>\w+)')
	COMMON   = re.compile(r'^common (?P<name>\w+)')
	INHERITS = re.compile(r'^inherits (?P<name>\w+)')
	OPENB    = re.compile(r'^{')
	VECTOR   = re.compile(r'^\s*(?P<name>\w+)')
	CLOSEB   = re.compile(r'^}')
	SID      = re.compile(r'^sid (?P<name>\w+)')
	EOF      = "end of file"

	#Constants used in header generation.
	USERSPACE = 0
	KERNEL    = 1

	CONSTANT_S = { \
		#parsing constants
		WHITE    : "whitespace", \
		COMMENT  : "comment", \
		USERFLAG : "userspace flag", \
		CLASS    : "class definition", \
		COMMON   : "common definition", \
		INHERITS : "inherits definition", \
		OPENB    : "'{'", \
		VECTOR   : "access vector definition", \
		CLOSEB   : "'}'", \
		SID      : "security identifier", \
		EOF      : "end of file", \
		#generation constants
		USERSPACE : "userspace mode", \
		KERNEL    : "kernel mode", \
	}

	def __init__(self):
		self.commons   = []
		self.user_commons = []
		self.common    = {}
		self.classes   = []
		self.vectors   = []
		self.vector    = {}
		self.userspace = {}
		self.sids      = []
		self.inherits  = {}
	
	def parseClasses(self, path):
		'''
		Parses security class definitions from the given path.
		'''
		classes = []
		input = open(path, 'r')

		number = 0
		for line in input:
			number += 1
			m = self.COMMENT.search(line)
			if m: continue

			m = self.WHITE.search(line)
			if m: continue

			m = self.CLASS.search(line)
			if m:
				g = m.groupdict()
				c = g['name']
				if c in classes: raise DuplicateError, (self.CLASS, path, number, c)
				classes.append(c)
				if self.USERFLAG.search(line):
					self.userspace[c] = True
				else:
					self.userspace[c] = False
				continue

			raise ParseError, ("data.  Was expecting either a comment, whitespace, or class definition. ", path, number)

		self.classes = classes
		return classes


	def parseSids(self, path):
		'''
		Parses initial SID definitions from the given path.
		'''
		sids = []
		input = open(path, 'r')
		for line in input:
			m = self.COMMENT.search(line)
			if m: continue

			m = self.WHITE.search(line)
			if m: continue

			m = self.SID.search(line)
			if m:
				g = m.groupdict()
				s = g['name']
				if s in sids: raise DuplicateError, (self.SID, path, number, s)
				sids.append(s)
				continue
			
			raise ParseError, ("data. Was expecting either a comment, whitespace, or security identifier. ", path, number)

		self.sids = sids
		return sids

	def parseVectors(self, path):
		'''
		Parses access vector definitions from the given path.
		'''
		vectors = []
		vector  = {}
		commons = []
		common = {}
		inherits = {}
		user_commons = {}
		input = open(path, 'r')

		# states
		NONE    = 0
		COMMON  = 1
		CLASS   = 2
		INHERIT = 3
		OPEN    = 4

		state = NONE
		state2 = NONE
		number = 0
		for line in input:
			number += 1
			m = self.COMMENT.search(line)
			if m: continue

			m = self.WHITE.search(line)
			if m: 
				if state == INHERIT:
					state = NONE
				continue

			m = self.COMMON.search(line)
			if m:
				if state != NONE: raise ParseError, (self.COMMON, path, number)
				g = m.groupdict()
				c = g['name']
				if c in commons: raise DuplicateError, (self.COMMON, path, number, c)
				commons.append(c)
				common[c] = []
				user_commons[c] = True
				state = COMMON
				continue

			m = self.CLASS.search(line)
			if m:
				if state != NONE: raise ParseError, (self.CLASS, number)
				g = m.groupdict()
				c = g['name']
				if c in vectors: raise DuplicateError, (self.CLASS, path, number, c)
				if c not in self.classes: raise UndefinedError, (self.CLASS, path, number, c)
				vectors.append(c)
				vector[c] = []
				state = CLASS
				continue
			
			m = self.INHERITS.search(line)
			if m:
				if state != CLASS: raise ParseError, (self.INHERITS, number)
				g = m.groupdict()
				i = g['name']
				if c in inherits: raise DuplicateError, (self.INHERITS, path, number, c)
				if i not in common: raise UndefinedError, (self.COMMON, path, number, i)
				inherits[c] = i
				state = INHERIT
				if not self.userspace[c]: user_commons[i] = False
				continue

			m = self.OPENB.search(line)
			if m:
				if (state != CLASS \
				and state != INHERIT \
				and state != COMMON) \
				or state2 != NONE: 
					raise ParseError, (self.OPENB, path, number)
				state2 = OPEN
				continue

			m = self.VECTOR.search(line)
			if m:
				if state2 != OPEN: raise ParseError, (self.VECTOR, path, number)
				g = m.groupdict()
				v = g['name']
				if state == CLASS or state == INHERIT:
					if v in vector[c]: raise DuplicateError, (self.VECTOR, path, number, v)
					vector[c].append(v)
				elif state == COMMON:
					if v in common[c]: raise DuplicateError, (self.VECTOR, path, number, v)
					common[c].append(v)
				continue

			m = self.CLOSEB.search(line)
			if m:
				if state2 != OPEN: raise ParseError, (self.CLOSEB, path, number)
				state = NONE
				state2 = NONE
				c = None
				continue
			
			raise ParseError, ("data", path, number)

		if state != NONE and state2 != NONE: raise ParseError, (self.EOF, path, number)

		cvdiff = set(self.classes) - set(vectors)
		if cvdiff: raise UnusedError, "Not all security classes were used in access vectors: %s" % cvdiff # the inverse of this will be caught as an undefined class error

# These arrays contain the classes, permissions and initial SIDs after 
# processing the Reference Policy security_classes, access_vector and 
# initial_sids files. The buildXxxPolicy Routines will then output the 
# requested policy file.
		self.commons = commons
		self.user_commons = user_commons
		self.common = common
		self.vectors = vectors
		self.vector = vector
		self.inherits = inherits
		return vector


	def createPolicyHdr(self, mode = USERSPACE):
		'''
		Creates a header file for processing by the libselinux examples.
		'''
		results = []
		results.append("/*\n * libselinux header file for selinux_set_mapping(3)\n")

		if mode == self.KERNEL:
			results.append(" * containing Kernel classes and permissions only\n */\n")
		else:
			results.append(" * containing Kernel + Userspace classes and permissions\n */\n")

#	print out the class + permissions
		results.append("static struct security_class_mapping map[] = {\n\n")

		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("\t{ \"%s\", { " % c)
				count = 0
				ps = []
				if self.inherits.has_key(c):
					ps += self.common[self.inherits[c]]
				ps += self.vector[c]
				for p in ps: 
					results.append("\"%s\", " % ps[count])
					count += 1
				results.append("NULL } },\n\n")
		results.append("\t{ NULL }\n};\n")

		return results

#
# Create a CIL Policy
#
	def createCilPolicy(self, mode = USERSPACE):
		'''
		Creates a CIL language formatted policy file for processing by the CIL compiler.
		'''
		results = []
		results.append(";\n; CIL policy ")

		if mode == self.KERNEL:
			results.append("- Kernel objects only\n;\n")
		else:
			results.append("- Kernel + Userspace objects\n;\n")

		results.append("; This policy has been generated using the Reference Policy\n; flask files: initial_sids, security_classes and access_vectors files\n; from: %s\n;\n" % (refpol))

#
# Set up the required default MLS information. This will be discarded 
# by the CIL policy build process for a non-MLS policy. It will
# allow the generation of a suitable default context.
#
		results.append("(category c0)\n")
		results.append("(categoryorder (c0))\n")
		results.append("(sensitivity s0)\n")
		results.append("(dominance (s0))\n")
		results.append("(sensitivitycategory s0 (c0))\n")
		results.append("(levelrange default ((s0 (c0)) (s0 (c0))))\n")
		results.append("(level low (s0 (c0)))\n\n")

#	 print out the sids 
		count = 0
		for s in self.sids:
			count += 1
			results.append("(sid %s)\n" % (s))
		results.append("\n")

#	print out the class + permissions
		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("(class %s ( " % c)
				count = 0
				ps = []
				if self.inherits.has_key(c):
					ps += self.common[self.inherits[c]]
				ps += self.vector[c]
				for p in ps: 
					results.append("%s " % ps[count])
					count += 1
				results.append("))\n")
		results.append("\n")

#	types and roles
		results.append("(type  %s)\n" % (se_type))
		results.append("(role  %s)\n" % (se_role))
		results.append("(roletype %s %s)\n\n" % (se_role, se_type))
		results.append("(role  %s)\n" % (obj_role))
		results.append("(roletype %s %s)\n\n" % (obj_role, se_type))

#	print out allow rules
		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("(allow %s self (%s ( " % (se_type, c))
				count = 0
				ps = []
				if self.inherits.has_key(c):
					ps += self.common[self.inherits[c]]
				ps += self.vector[c]
				for p in ps: 
					results.append("%s " % ps[count])
					count += 1
				results.append(")))\n")
		results.append("\n")

#	print out user statements
		results.append("(user  %s)\n" % se_user)
		results.append("(userrole  %s  %s)\n" % (se_user, se_role))
		results.append("(userrange %s (low low))\n" % se_user)
		results.append("(userlevel %s low)\n\n" % se_user)

		results.append("\n; Define SELinux user:\n")
		results.append("(selinuxuser name %s default)\n" % se_user)
		results.append("(selinuxuserdefault %s default)\n\n" % se_user)

		results.append("(user  %s)\n" % sys_user)
		results.append("(userrole  %s  %s)\n" % (sys_user, se_role))
		results.append("(userrole  %s  %s)\n" % (sys_user, obj_role))
		results.append("(userrange %s (low low))\n" % sys_user)
		results.append("(userlevel %s low)\n\n" % sys_user)

#	The context strings
		results.append("(context default_context (%s %s %s default))\n" % (se_user, se_role, se_type))
		results.append("(context system_context (%s %s %s default))\n" % (sys_user, se_role, se_type))
		results.append("(context object_context (%s %s %s default))\n\n" % (sys_user, obj_role, se_type))

#	Initial sids with contexts
		count = 0
		for s in self.sids:
			count += 1
			if s == "kernel":
				results.append("(sidcontext %s system_context)\n" % (s))
			else:
				results.append("(sidcontext %s object_context)\n" % (s))
		results.append("\n")

		results.append("(fsuse xattr ext2 object_context)\n")
		results.append("(fsuse xattr ext3 object_context)\n")
		results.append("(fsuse xattr ext4 object_context)\n")
		results.append("(fsuse xattr jfs object_context)\n")
		results.append("(fsuse xattr xfs object_context)\n")
		results.append("(fsuse xattr reiserfs object_context)\n")
		results.append("(fsuse xattr jffs2 object_context)\n")
		results.append("(fsuse xattr gfs2 object_context)\n")
		results.append("(fsuse xattr lustre object_context)\n")

		results.append("(fsuse task eventpollfs object_context)\n")
		results.append("(fsuse task pipefs object_context)\n")
		results.append("(fsuse task sockfs object_context)\n")

		results.append("(fsuse trans mqueue object_context)\n")
		results.append("(fsuse trans devpts object_context)\n")
		results.append("(fsuse trans hugetlbfs object_context)\n")
		results.append("(fsuse trans tmpfs object_context)\n")
		results.append("(fsuse trans shm object_context)\n\n")

		results.append("(genfscon selinuxfs / object_context)\n")
		results.append("(genfscon proc / object_context)\n\n")

#		CIL outputs these filecon statements, however currently use file_contexts file for demo.
		results.append("(filecon \"/\" \"\" any object_context)\n")
		results.append("(filecon \"/\" \".*\" any object_context)\n\n")

		return results

#
# Create a Policy Language Policy
#
	def createPolicy(self, mode = USERSPACE):
		'''
		Creates a non-MLS or MLS Policy Langauge file for processing by checkpolicy
		'''
		results = []

		if mls:
			results.append("#\n# MLS Policy Language policy ")
		else:
			results.append("#\n# Policy Language policy ")

		if mode == self.KERNEL:
			results.append("- Kernel objects only\n#\n")
		else:
			results.append("- Kernel + Userspace objects\n#\n")

#	print out the classes
		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("class %s\n" % c)

		results.append("\n")

#	 print out the sids 
		count = 0
		for s in self.sids:
			count += 1
			results.append("sid %s\n" % (s))
		results.append("\n")

#	print out the class + permissions
		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("class %s { " % c)
				count = 0
				ps = []
				if self.inherits.has_key(c):
					ps += self.common[self.inherits[c]]
				ps += self.vector[c]
				for p in ps: 
					results.append("%s " % ps[count])
					count += 1
				results.append("}\n")
		results.append("\n")

#	Check for MLS policy
		if mls:
			results.append("sensitivity s0;\n")
			results.append("dominance { s0 }\n")
			results.append("category c0;\n")
			results.append("level s0:c0;\n")
			results.append("# checkpolicy always wants an mlsconstrain or "
				"fails to build.\n#This is a simple one:\n")
			results.append("mlsconstrain { filesystem } { relabelto } ( h1 dom h2 );\n\n")

#	Example policycap for demo
		results.append("#\n# This policycap statement will be used in a "
				"netlabel module exercise\n# to show network_peer_controls. "
				"For now comment out:\npolicycap network_peer_controls;\n\n")

#	types and roles
		results.append("type  %s;\n" % (se_type))
		results.append("role  %s;\n" % (se_role))
		results.append("role %s types { %s };\n\n" % (se_role, se_type))
#		results.append("(role  %s)\n\n" % (obj_role))

#	print out allow rules
		for c in self.classes:
			if mode == self.KERNEL and self.userspace[c]:
				continue
			else:
				results.append("allow %s %s:%s { " % (se_type, se_type, c))
				count = 0
				ps = []
				if self.inherits.has_key(c):
					ps += self.common[self.inherits[c]]
				ps += self.vector[c]
				for p in ps: 
					results.append("%s " % ps[count])
					count += 1
				results.append("};\n")
		results.append("\n")

#	print out user statements
		if mls:
			results.append("user %s roles { %s } level s0 range s0;\n" % (se_user, se_role))
			results.append("user %s roles { %s } level s0 range s0;\n" % (sys_user, se_role))
		else:
			results.append("user %s roles { %s };\n" % (se_user, se_role))
			results.append("user %s roles { %s };\n" % (sys_user, se_role))
		results.append("\n");

#	An example constrain for demo
		results.append("#\n# This role constraint statement will be used to show "
			"limiting\n# a role transition in the external gateway. For now "
			"comment out:\nconstrain process transition (r1 == r2);\n\n")

		if mls:
#			Initial sids with contexts
			count = 0
			for s in self.sids:
				count += 1
				if s == "kernel":
					results.append("sid %s %s:%s:%s:s0\n" % (s, sys_user, se_role, se_type))
				else:
					results.append("sid %s %s:%s:%s:s0\n" % (s, sys_user, obj_role, se_type))

			results.append("\n")

			results.append("fs_use_xattr ext2 %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr ext3 %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr ext4 %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr jfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr xfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr reiserfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr jffs2 %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr gfs2 %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr lustre %s:%s:%s:s0;\n\n" % (sys_user, obj_role, se_type))
	
			results.append("fs_use_task eventpollfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_task pipefs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_task sockfs %s:%s:%s:s0;\n\n" % (sys_user, obj_role, se_type))

			results.append("fs_use_trans mqueue %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans devpts %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans hugetlbfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans tmpfs %s:%s:%s:s0;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans shm %s:%s:%s:s0;\n\n" % (sys_user, obj_role, se_type))
			results.append("genfscon selinuxfs / %s:%s:%s:s0\n\n" % (sys_user, obj_role, se_type))
			results.append("genfscon proc / %s:%s:%s:s0\n\n" % (sys_user, obj_role, se_type))

		else:
#			 default sids 
			count = 0
			for s in self.sids:
				count += 1
				if s == "kernel":
					results.append("sid %s %s:%s:%s\n" % (s, sys_user, se_role, se_type))
				else:
					results.append("sid %s %s:%s:%s\n" % (s, sys_user, obj_role, se_type))
			results.append("\n")


			results.append("fs_use_xattr ext2 %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr ext3 %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr ext4 %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr jfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr xfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr reiserfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr jffs2 %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr gfs2 %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_xattr lustre %s:%s:%s;\n\n" % (sys_user, obj_role, se_type))
	
			results.append("fs_use_task eventpollfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_task pipefs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_task sockfs %s:%s:%s;\n\n" % (sys_user, obj_role, se_type))

			results.append("fs_use_trans mqueue %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans devpts %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans hugetlbfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans tmpfs %s:%s:%s;\n" % (sys_user, obj_role, se_type))
			results.append("fs_use_trans shm %s:%s:%s;\n\n" % (sys_user, obj_role, se_type))
			results.append("genfscon selinuxfs / %s:%s:%s\n\n" % (sys_user, obj_role, se_type))
			results.append("genfscon proc / %s:%s:%s\n\n" % (sys_user, obj_role, se_type))

		return results




def usage():
	'''
	Returns the usage string.
	'''
	usage  = 'Usage: %s [-k] [-c|-M|-i] -r refpol_base -o output_file\n' % os.path.basename(sys.argv[0])
	usage += '\n'
	usage += ' -k\toutput kernel classes only (exclude # userspace entries in the\n\tsecurity_classes file)\n'
	usage += ' -c\toutput a policy in CIL language (this will contain MLS statements)\n'
	usage += ' -M\toutput an MLS policy using policy language statements\n'
	usage += ' -i\toutput a header file containing class/permissions for use by\n\tselinux_set_mapping(3)\n'
	usage += ' -r\tReference Policy source base directory (the path to the initial_sids,\n\tsecurity_classes and access_vectors files will be added from this base)\n'
	usage += ' -o\toutput file containing the generated policy or header file\n'
	usage += '\n\tIf no -c or -M options are given a non-MLS policy using policy language\n\tstatements will be generated that can be built using checkpolicy\n'

	return usage

########## MAIN ##########
if __name__ == '__main__':
	
	# Parse command line args
	try:
		opts, args = getopt.getopt(sys.argv[1:], 'o:r:Mckhi', ['MLS=', 'CIL=', 'policy=', 'kernel', 'help', 'include'])
	except getopt.GetoptError:
		print(usage())
		sys.exit(2)
	
	mls = 0
	cil = 0
	include = 0
	refpol = None
	outf = None
	mode = None
	for o, a in opts:
		if o in ('-h'):
			print(usage())
			sys.exit(0)
		elif o in ('-M'):
			mls = 1
		elif o in ('-c'):
			cil = 1
		elif o in ('-i'):
			include = 1
		elif o in ('-o'):
			outf = a
		elif o in ('-r'):
			refpol = a
		elif o in ('-k'):
			mode = Flask.KERNEL
		else:
			print(usage())
			sys.exit(2)

	if	refpol == None or \
		outf == None:
		print(usage())
		sys.exit(2)

	try:
		f = Flask()
		f.parseSids(refpol + "/policy/flask/initial_sids")
		f.parseClasses(refpol + "/policy/flask/security_classes")
		f.parseVectors(refpol +"/policy/flask/access_vectors")

	# Open the file and then create the requested policy source file
		of = open(outf, 'w')
		if cil == 1:
			of.writelines(f.createCilPolicy(mode))
		elif include == 1:
			of.writelines(f.createPolicyHdr(mode))
		else:
			of.writelines(f.createPolicy(mode))
		of.close()

	except Exception, e:
		print(e)
		sys.exit(2)
